

# parent_dir = D:\py_proj_2017\DigiPlot\digiplot
target_dir = r"backup"

file_typeD = {} # file type description. index=resource path, value=mime type
file_typeD[r"sample_poly.png"]="('image/png', None)"
