(lp1
(dp2
S'y'
I68
sS'x'
I391
sS'height'
p3
I546
sS'name'
p4
Vtemp
p5
sS'width'
p6
I555
sa(dp7
a(dp8
S'hasmenu'
p9
S'yes'
p10
sS'hasstatusbar'
p11
S'yes'
p12
sS'guitype'
p13
S'main'
p14
sS'hideOkBtn'
p15
S'no'
p16
sS'maxCompID'
p17
I3
sS'hasstddialcolor'
p18
S'no'
p19
sS'hasstddialmess'
p20
S'yes'
p21
sS'resizable'
p22
S'yes'
p23
sS'menu'
p24
VFile\u000a  Import Image\u000a  Save CSV\u000aAnchor Plot\u000a  Set Xmin\u000a  Set Xmax\u000a  Set Ymin\u000a  Set Ymax\u000aHelp\u000aExit\u000a  Quit
p25
sS'hasstdalarm'
p26
S'no'
p27
sS'hasstddialfile'
p28
S'yes'
p29
sa.(lp1
(lp2
S'canvas'
p3
a(dp4
S'name'
p5
S'Plot'
p6
sS'compID'
p7
I1
sS'text'
p8
S'Canvas_1 text'
p9
sS'scrolly'
p10
S'no'
p11
sS'y'
I36
sS'x'
I216
sa(dp12
S'width'
p13
I313
sS'height'
p14
I362
saa(lp15
S'listbox'
p16
a(dp17
g5
S'Defined_Points'
p18
sg7
I2
sg8
S'Listbox_1 text'
p19
sg10
S'yes'
p20
sS'pixelWidth'
p21
I179
sS'y'
I36
sS'x'
I24
sS'pixelHeight'
p22
I365
sa(dp23
g13
I15
sS'selectmode'
p24
S'extended'
p25
saa(lp26
S'label'
p27
a(dp28
g5
S'Defined_Points'
p29
sg7
I3
sg21
I112
sS'y'
I12
sS'x'
I48
sg22
I22
sa(dp30
g8
S'Defined Points'
p31
sg13
I15
saa.